/*
	Copyright 1985, 1986, 1987, 1988, 1989, 1990, 1991 Chris Lewis
		All Rights Reserved

    See the LICENSE file for a full description of restrictions under which
    this software is provided.

    Function:		ditroff frontend.
 */

#include "defs.h"

#ifndef	lint
static char SCCSid[] =
    "@(#)dit.c: 91/05/30 Copyright 91/05/30 01:45:57 Chris Lewis";
#endif

extern struct cattab tabN[], tabS[], *extidx;
extern struct troff2befont *extchar;

/*	Translation of single character characters to cattab
	entries and, hence, emittable strings via the backends.
 */
struct cattab **dittab;

/*	\(xx characters */
struct cattab **spctab, **ts;

struct cattab *ditsearch(str)
char *str; {
    register int l, mid, h, k;
    struct cattab *dp;
    l = 0;
    h = extcount - 1;
#ifdef	NEVER
    for (ts = spctab; *ts; ts++)
	if (strcmp((*ts)->ch_name, str) == 0)
	    return(*ts);
#endif
    while(l <= h) {
	mid = (l + h) >> 1;
	k = strcmp(str, spctab[mid]->ch_name);
	if (k == 0) {
	    dp = spctab[mid];
	    DBP((D_CAT,
		"D ditsearch(%s) -> ch_name: %s, set: %d, catidx: %d, wididx: %d, desc: %s\n",
		str, dp->ch_name, dp->ch_set, dp->ch_catidx, dp->ch_wididx, dp->ch_desc));
	    return(dp);
	} else if (k < 0)
	    h = mid - 1;
	else
	    l = mid + 1;
    }
    return((struct cattab *) NULL);
}

catcmp(a, b)
struct cattab **a, **b; {
    return(strcmp((*a)->ch_name, (*b)->ch_name));
}

addtab(table)
struct cattab *table; {
    register struct cattab *p;

    for (p = table; p->ch_name != NOC; p++) {
	if (p->ch_desc != NOC && p->ch_catidx != NTC) {
	    DBP((D_CAT, "D addtab: %s, desc: %s, catidx: %d\n", p->ch_name,
		p->ch_desc, p->ch_catidx));
	    if (strlen(p->ch_name) == 1) {
		dittab[p->ch_name[0]] = p;
	    } else {
		if (extcount > 0 && !(extcount%EXTCHUNK)) {
		    spctab = (struct cattab **) realloc(spctab,
			sizeof(struct cattab *) * (extcount + EXTCHUNK + 1));
		    clrarray(&spctab[extcount], sizeof(struct cattab *) *
			(EXTCHUNK+1));
		}
		spctab[extcount++] = p;
	    }
	}
    }
}

fixtab() {
    register int i;

    dittab = (struct cattab **) mustmalloc(sizeof(struct cattab *) * 256,
	"dittab");

    spctab = (struct cattab **) mustmalloc(sizeof(struct cattab *) *
	(EXTCHUNK+1), "spctab");

    extcount = 0;

    addtab(tabN);
    addtab(tabS);
    if (extidx)
	addtab(extidx);

    /*	Magic */
    dittab['-'] = &tabN[31];
    dittab['_'] = &tabS[31];

    DBP((D_CAT, "D end addtab: %d characters\n", extcount));

    spctab[extcount] = NULL;

    qsort(spctab, extcount, sizeof (struct cattab *), catcmp);

#ifdef DEBUG
    if (debug&D_CAT) {
	int i;

	DBP((D_CAT, "D after qsort\n"));

	for (ts = spctab; *ts; ts++)
	    DBP((D_CAT, "D %s -> %08x (%d)\n", (*ts)->ch_name, *ts,
		(*ts)->ch_catidx));

	for (i = 0; i < 256; i++)
	    if (dittab[i])
		DBP((D_CAT, "D %d:%02x:%c: %d/%d\n", i, i, i, dittab[i]->ch_set,
		dittab[i]->ch_catidx));
    }
#endif
}

#ifdef	DIT

int sizescale = 1;
int indtres = 300;
int halfdtres = 150;

#define	DIT2CAT(x)	(int) (((long) (x) * TROFFRESOLUTION + halfdtres) / indtres)

int points, font, ch, i;
int xpos, ypos;

dit() {
    register int cmd, nc, i, j;
    register struct cattab *p;
    register struct cattab *last = (struct cattab *) NULL;
    char special[512];
    fixtab();
    cmd = skipwhite();
    if (cmd == 0x40) {
	fprintf(stderr, "%s: illegal starting sequence for ditroff.\n",
	    progname);
	fprintf(stderr, "I think this is really CAT troff output.\n");
	fprintf(stderr, "Please double check your troff & -N settings.\n");
	exit(1);
    }
    while(1) {
	switch(cmd) {
	    default:
		fprintf(stderr, "%s: illegal command 0x%02x.\n", progname, cmd);
		break;
		/*NOTREACHED*/
	    case EOF:
		goto finish;
	    case 's':
		points = getnum();
		points /= sizescale;
		DBP((D_CAT, "D point: %d\n", points));
		break;
	    case 'f':
		font = getnum();
		if (font < 0 || font >= INTFONTS) {
		    fprintf(stderr,
			"%s: font number %d too high - get help from Chris!\n",
			progname, font);
		    font = 1;
		} else if (font == 0)
		    /* font 0 stuff - because I map font n to n-1 internally */
		    font = INTFONTS - 1;
		else
		    font--;
		DBP((D_CAT, "D font: %d\n", font));
		break;
	    case '0':
	    case '1':
	    case '2':
	    case '3':
	    case '4':
	    case '5':
	    case '6':
	    case '7':
	    case '8':
	    case '9':
		xpos += (cmd - '0') * 10 + (getchar() - '0');
		/*fallthru*/
	    case 'c':
		p = dittab[ch = (getchar()&0xff)];
		if (!p) {
		    char seq[2];

		    if (ch == ' ')
			break;
		    fprintf(stderr,
			"%s: don't know this character: 0x%02x(%c)\n",
			progname, ch, ch);
		    seq[0] = ch;
		    seq[1] = '\0';
		    ditemit(xpos, ypos, font, points, 0, seq);
		    break;
		}
		if (p->ch_wididx == NTC) /* extended character */
		    ditemit(xpos, ypos,
			p->ch_set == N ? -font-1: -symidx-1,
			points, p->ch_catidx,
			extchars[p->ch_catidx].t2b_charseq);
		else
		    ditemit(xpos, ypos,
			p->ch_set == N ? font: symidx,
			points, p->ch_catidx, (char *) NULL);
		break;

	    case 'u':		/* groff kern this sequence command */
		j = getnum();
		/* kern ignored for the moment - have to read the width
		   table */
		/* fallthru */
	    case 't':		/* groff sequence */
		{ char ubuf[100];
		    /* should really read the width tables and
		       do each character individually, but for the
		       moment, this should work reasonably well */
#ifdef	OPT
		    canonflush();
#endif
		    while((ch = getchar()) != EOF && isspace(ch));
		    if (ch == EOF)
			break;
		    ubuf[i++] = ch;
		    while((ch = getchar()) != EOF && !isspace(ch))
			ubuf[i++] = ch;
		    p = dittab[ubuf[0]&0xff];
		    if (!p)
			ditemit(xpos, ypos, font, points, 0, ubuf);
		    else if (p->ch_wididx == NTC)
			ditemit(xpos, ypos,
			    p->ch_set == N ? -font-1: -symidx-1,
			    points, 0, ubuf);
		    else
			ditemit(xpos, ypos, p->ch_set == N ? font: symidx,
			    points, 0, ubuf);
		}
	    case 'C':
		i = 0;
		while((ch = getchar()) != EOF && !isspace(ch))
		    special[i++] = ch;
		special[i] = '\0';
		DBP((D_CAT, "D C%s, font: %d\n", special, font));
		if (last && strcmp(last->ch_name, special) == 0)
		    p = last;
		else {
		    p = ditsearch(special);
		    last = p;
		}
		if (!p) {
		    fprintf(stderr,
			"%s: don't know this character: %s\n", progname,
			special);
		} else {
		    if (p->ch_wididx == NTC) /* extended character */
			ditemit(xpos, ypos,
			    p->ch_set == N ? -font-1 : -symidx-1,
			    points, p->ch_catidx,
			    extchars[p->ch_catidx].t2b_charseq);
		    else {
			int mfont = font;
			if (font == symidx && p->ch_set == N)
			    mfont = 0;
			else if (font != symidx && p->ch_set == S)
			    mfont = symidx;
			ditemit(xpos, ypos, mfont, points, p->ch_catidx,
			    (char *) NULL);
		    }
		}
		break;
	    case 'x':
		{
		    char cmd[20], a1[20], a2[20], a3[20];
		    int cnt;
		    i = 0;
#ifdef	OPT
		    canonflush();
#endif
		    /* synchronize special X & Y to current position
		       (ie: so psfig will work) */
		    specXPos = DIT2CAT(xpos);
		    specYPos = DIT2CAT(ypos);

		    while((ch = getchar()) != EOF && isspace(ch));
		    special[i++] = ch;
		    while((ch = getchar()) != EOF && ch != '\n')
			special[i++] = ch;
		    special[i] = '\0';
		    DBP((D_CAT, "D x: %s\n", special));
		    i = sscanf(special, "%s %s %s %s", cmd, a1, a2, a3);
		    if (i <= 0) {
			fprintf(stderr, "%s: bad x command: %s\n",
			    progname, special);
			exit(1);
		    }
		    switch(cmd[0]) {
			case 'p':
			case 'H':
			case 'S':
			    break;
			case 'f':	/* font load */
			    if (i != 3) {
				fprintf(stderr, "%s: bad font command: %s\n",
				progname, special);
				exit(1);
			    }
			    sprintf(special, "F%s%s", a1, a2);
			    DBP((D_CAT, "D load: %s %s\n", a1, a2));

			    if (be->befontsel)
				(*be->befontsel)(a1,a2);

			    break;
			case 'r':	/* specify resolution */
			    if (i != 4) {
				fprintf(stderr, "%s: bad res command: %s\n",
				progname, special);
				exit(1);
			    }
			    indtres = atoi(a1);
			    if (indtres <= 0) {
				fprintf(stderr,
				    "%s: ridiculous res value: %s\n",
				    progname, special);
				exit(1);
			    }
			    halfdtres = indtres / 2;
			    DBP((D_CAT, "D res: %d\n", indtres));
			    break;
			case 'i':
			    DBP((D_CAT, "D init\n"));
			    if (be->beprolog)
				(*be->beprolog)();
			    resetState();
			    break;
			case 't':
			    DBP((D_CAT, "D trailer\n"));
			    break;
			case 's':
			    DBP((D_CAT, "D stop\n"));
			    return;
			case 'T':
			    if (i != 2) {
				fprintf(stderr, "%s: bad x T command: %s\n",
				    progname, special);
				exit(1);
			    }
			    device = mustmalloc(strlen(a1) + 1, "device");
			    strcpy(device, a1);
			    DBP((D_CAT, "D type: %s\n", a1));
			    break;
			case 'X':
			    switch(a1[0]) {
				case 'f':
				    a1[0] = 'I';
				    break;
				case 'p':
				    a1[0] = 'P';
				    break;
			    }
			    dospecial(a1);
			    break;

			default:
			    fprintf(stderr, "%s: unknown special: %s\n",
				progname, special);
		    }
		}
		break;
	    case 'H':
		xpos = getnum();
		DBP((D_CAT, "D H: %d\n", xpos));
		break;
	    case 'h':
		xpos += getnum();
		DBP((D_CAT, "D Hi: %d\n", xpos));
		break;
	    case 'V':
#ifdef	OPT
		canonflush();
#endif
		ypos = getnum();
		DBP((D_CAT, "D V: %d\n", ypos));
		break;
	    case 'v':
#ifdef	OPT
		canonflush();
#endif
		ypos += getnum();
		DBP((D_CAT, "D Vi: %d\n", ypos));
		break;
		break;
	    case 'p':
		ch = getnum();
		DBP((D_CAT, "D page: %d\n", ch));
		if (be->bepage)
		    (*be->bepage)();
		break;
	    case 'n':	/* end of line */
		ch = getnum();
		ch = skipwhite();
		ungetc(ch, stdin);
		ch = getnum();
		break;
	    case 'i':	/* stipple? BERK */
		ch = getnum();
		break;
	    case 'P':	/* split end? BERK */
		break;
	    case 'w':
		break;
	    case '#':
	    case 'D':
	    case '!':
#ifdef	OPT
		canonflush();
#endif
		i = 1;
		special[0] = cmd;
		while((ch = getchar()) != EOF && ch != '\n')
		    special[i++] = ch;
		special[i] = '\0';
		DBP((D_CAT, "D passthru: %s\n", special));

		switch(cmd) {
		    case '#':
			break;
		    case '!':
			special[0] = 'P';
			dospecial(special);
			break;
		    case 'D': {
			short values[100];
			int opcode, ct, newx, newy;

			if (!(opcode = drawparse(&ct, values, &special[1])))
			    break;
			if (!strchr("lcCeEa~tfpP", opcode)) {
			    fprintf(stderr, "%s: unknown draw code %s\n",
				progname, special);
			    break;
			}
			newx = xpos;
			newy = ypos;
			if (opcode == 'e' || opcode == 'E')
			    newx += values[0];
			else if (opcode != 't' && opcode != 'f') {
			    for (i = 0; i < ct/2; i++) {
				newx += values[i*2];
				newy += values[i*2+1];
			    }
			    if (i*2 < ct)
				newx += values[i*2];
			}
			if (opcode != 't' && opcode != 'f')
			    for (i = 0; i < ct; i++)
				values[i] = DIT2CAT(values[i]);

			DBP((D_CAT, "D D: xpos,ypos,opcode,ct,special = %d,%d,%d,%d,%s\n",
			    xpos, ypos, opcode, ct, special));

			if (be->bedraw)
			    (*be->bedraw)(DIT2CAT(xpos), DIT2CAT(ypos),
					 opcode, ct, values, special);
			xpos = newx;
			ypos = newy;
		    }
		}
		break;
	}
	cmd = skipwhite();
    }
    finish: ;
}

skipwhite() {
    int c;
    while((c = getchar()) != EOF && isspace(c));
    return(c);
}

getnum() {
    register int ret = 0;
    int c;
    while((c = getchar()) != EOF && isdigit(c))
	ret = ret * 10 + (c - '0');
    ungetc(c, stdin);
    return(ret);
}

ditemit(x, y, font, points, troffChar, sequence)
int x, y;
int font, points, troffChar;
register char *sequence; {
    DBP((D_CAT, "D E: x,y: %d/%d -> ", x, y));
    x = DIT2CAT(x);
    y = DIT2CAT(y);
    DBP((D_CAT, "%d/%d (font,points,ch = %d,%d,%d)\n", x, y, font, points,
	troffChar));
#ifdef OPT
    if (sequence) {
	canonflush();
	if (be->beputchar)
	    (*be->beputchar)(x, y, font, points, troffChar, sequence);
    } else
	canoninsert(x, y, font, points, troffChar);
#else
    if (be->beputchar)
	(*be->beputchar)(x, y, font, points, troffChar, sequence);
#endif
}
#endif

